@@ -14,7 +14,8 @@ from itertools import product
 import warnings
 
 import numpy as np
-from scipy.ndimage import gaussian_filter
+from scipy.ndimage import gaussian_filter, zoom as scipy_zoom
+import xarray as xr
 
 from .. import constants as mpconsts
 from ..package_tools import Exporter
@@ -52,8 +53,7 @@ def wind_speed(u, v):
     wind_components
 
     """
-    speed = np.sqrt(u * u + v * v)
-    return speed
+    return np.hypot(u, v)
 
 
 @exporter.export
@@ -99,11 +99,12 @@ def wind_direction(u, v, convention='from'):
     elif convention not in ('to', 'from'):
         raise ValueError('Invalid kwarg for "convention". Valid options are "from" or "to".')
 
-    mask = wdir <= 0
+    mask = np.array(wdir <= 0)
     if np.any(mask):
         wdir[mask] += units.Quantity(360., 'deg')
     # avoid unintended modification of `pint.Quantity` by direct use of magnitude
-    calm_mask = (np.asarray(u.magnitude) == 0.) & (np.asarray(v.magnitude) == 0.)
+    calm_mask = (np.asanyarray(u.magnitude) == 0.) & (np.asanyarray(v.magnitude) == 0.)
+
     # np.any check required for legacy numpy which treats 0-d False boolean index as zero
     if np.any(calm_mask):
         wdir[calm_mask] = units.Quantity(0., 'deg')
@@ -259,10 +260,10 @@ def heat_index(temperature, relative_humidity, mask_undefined=True):
     if not hasattr(relative_humidity, 'units'):
         relative_humidity = units.Quantity(relative_humidity, 'dimensionless')
     delta = temperature.to(units.degF) - units.Quantity(0., 'degF')
-    rh2 = relative_humidity * relative_humidity
-    delta2 = delta * delta
+    rh2 = relative_humidity**2
+    delta2 = delta**2
 
-    # Simplifed Heat Index -- constants converted for relative_humidity in [0, 1]
+    # Simplified Heat Index -- constants converted for relative_humidity in [0, 1]
     a = (units.Quantity(-10.3, 'degF') + 1.1 * delta
          + units.Quantity(4.7, 'delta_degF') * relative_humidity)
 
@@ -284,12 +285,12 @@ def heat_index(temperature, relative_humidity, mask_undefined=True):
         hi = masked_array(hi)
 
     # If T <= 40F, Heat Index is T
-    sel = (temperature <= units.Quantity(40., 'degF'))
+    sel = np.array(temperature <= units.Quantity(40., 'degF'))
     if np.any(sel):
         hi[sel] = temperature[sel].to(units.degF)
 
     # If a < 79F and hi is unset, Heat Index is a
-    sel = (a < units.Quantity(79., 'degF')) & np.isnan(hi)
+    sel = np.array(a < units.Quantity(79., 'degF')) & np.isnan(hi)
     if np.any(sel):
         hi[sel] = a[sel]
 
@@ -299,9 +300,9 @@ def heat_index(temperature, relative_humidity, mask_undefined=True):
         hi[sel] = b[sel]
 
     # Adjustment for RH <= 13% and 80F <= T <= 112F
-    sel = ((relative_humidity <= units.Quantity(13., 'percent'))
-           & (temperature >= units.Quantity(80., 'degF'))
-           & (temperature <= units.Quantity(112., 'degF')))
+    sel = np.array((relative_humidity <= units.Quantity(13., 'percent'))
+                   & (temperature >= units.Quantity(80., 'degF'))
+                   & (temperature <= units.Quantity(112., 'degF')))
     if np.any(sel):
         rh15adj = ((13. - relative_humidity[sel] * 100.) / 4.
                    * np.sqrt((units.Quantity(17., 'delta_degF')
@@ -310,9 +311,9 @@ def heat_index(temperature, relative_humidity, mask_undefined=True):
         hi[sel] = hi[sel] - rh15adj
 
     # Adjustment for RH > 85% and 80F <= T <= 87F
-    sel = ((relative_humidity > units.Quantity(85., 'percent'))
-           & (temperature >= units.Quantity(80., 'degF'))
-           & (temperature <= units.Quantity(87., 'degF')))
+    sel = np.array((relative_humidity > units.Quantity(85., 'percent'))
+                   & (temperature >= units.Quantity(80., 'degF'))
+                   & (temperature <= units.Quantity(87., 'degF')))
     if np.any(sel):
         rh85adj = (0.02 * (relative_humidity[sel] * 100. - 85.)
                    * (units.Quantity(87., 'delta_degF') - delta[sel]))
@@ -323,6 +324,7 @@ def heat_index(temperature, relative_humidity, mask_undefined=True):
         mask = np.array(temperature < units.Quantity(80., 'degF'))
         if mask.any():
             hi = masked_array(hi, mask=mask)
+
     return hi
 
 
@@ -373,7 +375,7 @@ def apparent_temperature(temperature, relative_humidity, speed, face_level_winds
     heat_index, windchill
 
     """
-    is_not_scalar = isinstance(temperature.m, (list, tuple, np.ndarray))
+    is_not_scalar = hasattr(temperature, '__len__')
 
     temperature = np.atleast_1d(temperature)
     relative_humidity = np.atleast_1d(relative_humidity)
@@ -473,6 +475,10 @@ def height_to_geopotential(height):
     39201.98726524 48994.79863351 58784.53871501 68571.20895435
     78354.81079527 88135.34568058 97912.81505219], 'meter ** 2 / second ** 2')>
 
+    See Also
+    --------
+    geopotential_to_height
+
     Notes
     -----
     This calculation approximates :math:`g(z)` as
@@ -486,10 +492,6 @@ def height_to_geopotential(height):
     (Prior to MetPy v0.11, this formula instead calculated :math:`g(z)` from Newton's Law of
     Gravitation assuming a spherical Earth and no centrifugal force effects).
 
-    See Also
-    --------
-    geopotential_to_height
-
     """
     return (mpconsts.g * mpconsts.Re * height) / (mpconsts.Re + height)
 
@@ -535,6 +537,10 @@ def geopotential_to_height(geopotential):
     <Quantity([     0.   1000.   2000.   3000.   4000.   5000.   6000.   7000.   8000.
     9000.  10000.], 'meter')>
 
+    See Also
+    --------
+    height_to_geopotential
+
     Notes
     -----
     This calculation approximates :math:`g(z)` as
@@ -551,10 +557,6 @@ def geopotential_to_height(geopotential):
     .. versionchanged:: 1.0
        Renamed ``geopot`` parameter to ``geopotential``
 
-    See Also
-    --------
-    height_to_geopotential
-
     """
     return (geopotential * mpconsts.Re) / (mpconsts.g * mpconsts.Re - geopotential)
 
@@ -588,7 +590,7 @@ def height_to_pressure_std(height):
 @exporter.export
 @preprocess_and_wrap(wrap_like='latitude')
 def coriolis_parameter(latitude):
-    r"""Calculate the coriolis parameter at each point.
+    r"""Calculate the Coriolis parameter at each point.
 
     The implementation uses the formula outlined in [Hobbs1977]_ pg.370-371.
 
@@ -600,7 +602,7 @@ def coriolis_parameter(latitude):
     Returns
     -------
     `pint.Quantity`
-        Corresponding coriolis force at each point
+        Corresponding Coriolis force at each point
 
     """
     latitude = _check_radians(latitude, max_radians=np.pi / 2)
@@ -736,7 +738,7 @@ def smooth_gaussian(scalar_grid, n):
     but is not identical.  The following notes are incorporated from
     the GEMPAK source code:
 
-    This function smoothes a scalar grid using a moving average
+    This function smooths a scalar grid using a moving average
     low-pass filter whose weights are determined by the normal
     (Gaussian) probability distribution function for two dimensions.
     The weight given to any grid point within the area covered by the
@@ -771,7 +773,7 @@ def smooth_gaussian(scalar_grid, n):
     N has the effect of expanding the moving average window as well as
     changing the values of weights.  The patch is a square covering all
     points whose weight values are within two standard deviations of the
-    mean of the two dimensional normal distribution.
+    mean of the two-dimensional normal distribution.
 
     The key difference between GEMPAK's GWFS and this function is that,
     in GEMPAK, the leftover weight values representing the fringe of the
@@ -790,8 +792,7 @@ def smooth_gaussian(scalar_grid, n):
     """
     # Compute standard deviation in a manner consistent with GEMPAK
     n = int(round(n))
-    if n < 2:
-        n = 2
+    n = max(n, 2)
     sgma = n / (2 * np.pi)
 
     # Construct sigma sequence so smoothing occurs only in horizontal direction
@@ -799,8 +800,12 @@ def smooth_gaussian(scalar_grid, n):
     # Assume the last two axes represent the horizontal directions
     sgma_seq = [sgma if i > num_ax - 3 else 0 for i in range(num_ax)]
 
-    # Compute smoothed field
-    return gaussian_filter(scalar_grid, sgma_seq, truncate=2 * np.sqrt(2))
+    filter_args = {'sigma': sgma_seq, 'truncate': 2 * np.sqrt(2)}
+    if hasattr(scalar_grid, 'mask'):
+        smoothed = gaussian_filter(scalar_grid.data, **filter_args)
+        return np.ma.array(smoothed, mask=scalar_grid.mask)
+    else:
+        return gaussian_filter(scalar_grid, **filter_args)
 
 
 @exporter.export
@@ -831,6 +836,10 @@ def smooth_window(scalar_grid, window, passes=1, normalize_weights=True):
     array-like
         The filtered scalar grid
 
+    See Also
+    --------
+    smooth_rectangular, smooth_circular, smooth_n_point, smooth_gaussian
+
     Notes
     -----
     This function can be applied multiple times to create a more smoothed field and will only
@@ -841,10 +850,6 @@ def smooth_window(scalar_grid, window, passes=1, normalize_weights=True):
     Applying the smoothing function multiple times will propagate NaNs further throughout the
     domain.
 
-    See Also
-    --------
-    smooth_rectangular, smooth_circular, smooth_n_point, smooth_gaussian
-
     """
     def _pad(n):
         # Return number of entries to pad given length along dimension.
@@ -915,6 +920,10 @@ def smooth_rectangular(scalar_grid, size, passes=1):
     array-like
         The filtered scalar grid
 
+    See Also
+    --------
+    smooth_window, smooth_circular, smooth_n_point, smooth_gaussian
+
     Notes
     -----
     This function can be applied multiple times to create a more smoothed field and will only
@@ -924,10 +933,6 @@ def smooth_rectangular(scalar_grid, size, passes=1):
     point that uses that particular grid point in the smoothing calculation. Applying the
     smoothing function multiple times will propagate NaNs further throughout the domain.
 
-    See Also
-    --------
-    smooth_window, smooth_circular, smooth_n_point, smooth_gaussian
-
     """
     return smooth_window(scalar_grid, np.ones(size), passes=passes)
 
@@ -954,6 +959,10 @@ def smooth_circular(scalar_grid, radius, passes=1):
     array-like
         The filtered scalar grid
 
+    See Also
+    --------
+    smooth_window, smooth_rectangular, smooth_n_point, smooth_gaussian
+
     Notes
     -----
     This function can be applied multiple times to create a more smoothed field and will only
@@ -963,10 +972,6 @@ def smooth_circular(scalar_grid, radius, passes=1):
     particular grid point in the smoothing calculation. Applying the smoothing function
     multiple times will propagate NaNs further throughout the domain.
 
-    See Also
-    --------
-    smooth_window, smooth_rectangular, smooth_n_point, smooth_gaussian
-
     """
     # Generate the circle
     size = 2 * radius + 1
@@ -1000,6 +1005,10 @@ def smooth_n_point(scalar_grid, n=5, passes=1):
     array-like or `pint.Quantity`
         The filtered scalar grid
 
+    See Also
+    --------
+    smooth_window, smooth_rectangular, smooth_circular, smooth_gaussian
+
     Notes
     -----
     This function is a close replication of the GEMPAK function SM5S and SM9S depending on the
@@ -1011,10 +1020,6 @@ def smooth_n_point(scalar_grid, n=5, passes=1):
     Applying the smoothing function multiple times will propagate NaNs further throughout the
     domain.
 
-    See Also
-    --------
-    smooth_window, smooth_rectangular, smooth_circular, smooth_gaussian
-
     """
     if n == 9:
         weights = np.array([[0.0625, 0.125, 0.0625],
@@ -1031,6 +1036,68 @@ def smooth_n_point(scalar_grid, n=5, passes=1):
     return smooth_window(scalar_grid, window=weights, passes=passes, normalize_weights=False)
 
 
+@exporter.export
+def zoom_xarray(input_field, zoom, output=None, order=3, mode='constant', cval=0.0,
+                prefilter=True):
+    """Apply a spline interpolation to the data to effectively reduce the grid spacing.
+
+    This function applies `scipy.ndimage.zoom` to increase the number of grid points and
+    effectively reduce the grid spacing over the data provided.
+
+    Parameters
+    ----------
+    input_field  : `xarray.DataArray`
+        The 2D data array to be interpolated.
+
+    zoom : float or sequence
+        The zoom factor along the axes. If a float, zoom is the same for each axis. If a
+        sequence, zoom should contain one value for each axis.
+
+    order : int, optional
+        The order of the spline interpolation, default is 3. The order has to be in the
+        range 0-5.
+
+    mode : {‘reflect’, ‘grid-mirror’, ‘constant’, ‘grid-constant’, ‘nearest’, ‘mirror’,
+        ‘grid-wrap’, ‘wrap’}, optional
+        See `scipy.ndimage.zoom` documentation for details.
+
+    cval : scalar, optional
+        See `scipy.ndimage.zoom` documentation for details.
+
+    prefilter : bool (default = True)
+        See `scipy.ndimage.zoom` documentation for details.
+
+    Returns
+    -------
+    zoomed_data: `xarray.DataArray`
+        The zoomed input with its associated coordinates and coordinate reference system, if
+        available.
+
+    """
+    # Zoom data
+    zoomed_data = scipy_zoom(
+        input_field.data, zoom, output=output, order=order, mode=mode, cval=cval,
+        prefilter=prefilter
+    )
+
+    # Zoom dimension coordinates
+    if not np.iterable(zoom):
+        zoom = tuple(zoom for _ in input_field.dims)
+    zoomed_dim_coords = {}
+    for dim_name, dim_zoom in zip(input_field.dims, zoom):
+        if dim_name in input_field.coords:
+            zoomed_dim_coords[dim_name] = scipy_zoom(
+                input_field[dim_name].data, dim_zoom, order=order, mode=mode, cval=cval,
+                prefilter=prefilter
+            )
+    if hasattr(input_field, 'metpy_crs'):
+        zoomed_dim_coords['metpy_crs'] = input_field.metpy_crs
+    # Reconstruct (ignoring non-dimension coordinates)
+    return xr.DataArray(
+        zoomed_data, dims=input_field.dims, coords=zoomed_dim_coords, attrs=input_field.attrs
+    )
+
+
 @exporter.export
 @preprocess_and_wrap(wrap_like='altimeter_value')
 @check_units('[pressure]', '[length]')
