@@ -138,16 +138,16 @@ def natural_neighbor_to_grid(xp, yp, variable, grid_x, grid_y):
 
     Parameters
     ----------
-    xp: (N, ) ndarray
+    xp: (P, ) ndarray
         x-coordinates of observations
-    yp: (N, ) ndarray
+    yp: (P, ) ndarray
         y-coordinates of observations
-    variable: (N, ) ndarray
+    variable: (P, ) ndarray
         observation values associated with (xp, yp) pairs.
         IE, variable[i] is a unique observation at (xp[i], yp[i])
-    grid_x: (M, 2) ndarray
+    grid_x: (M, N) ndarray
         Meshgrid associated with x dimension
-    grid_y: (M, 2) ndarray
+    grid_y: (M, N) ndarray
         Meshgrid associated with y dimension
 
     Returns
@@ -245,7 +245,7 @@ def interpolate_to_grid(x, y, z, interp_type='linear', hres=50000,
         The horizontal resolution of the generated grid, given in the same units as the
         x and y parameters. Default 50000.
     minimum_neighbors: int
-        Minimum number of neighbors needed to perform barnes or cressman interpolation for a
+        Minimum number of neighbors needed to perform Barnes or Cressman interpolation for a
         point. Default is 3.
     gamma: float
         Adjustable smoothing parameter for the barnes interpolation. Default 0.25.
@@ -253,8 +253,8 @@ def interpolate_to_grid(x, y, z, interp_type='linear', hres=50000,
         Response parameter for barnes interpolation, specified nondimensionally
         in terms of the Nyquist. Default 5.052
     search_radius: float
-        A search radius to use for the barnes and cressman interpolation schemes.
-        If search_radius is not specified, it will default to the average spacing of
+        A search radius to use for the Barnes and Cressman interpolation schemes.
+        If search_radius is not specified, it will default to 5 times the average spacing of
         observations.
     rbf_func: str
         Specifies which function to use for Rbf interpolation.
@@ -276,6 +276,10 @@ def interpolate_to_grid(x, y, z, interp_type='linear', hres=50000,
     img: (M, N) ndarray
         2-dimensional array representing the interpolated values for each grid.
 
+    See Also
+    --------
+    interpolate_to_points
+
     Notes
     -----
     This function acts as a wrapper for `interpolate_points` to allow it to generate a regular
@@ -284,10 +288,6 @@ def interpolate_to_grid(x, y, z, interp_type='linear', hres=50000,
     This function interpolates points to a Cartesian plane, even if lat/lon coordinates
     are provided.
 
-    See Also
-    --------
-    interpolate_to_points
-
     """
     # Generate the grid
     if boundary_coords is None:
