@@ -11,7 +11,7 @@ import warnings
 import numpy as np
 import pandas as pd
 
-from ._metar_parser.metar_parser import parse, ParseError
+from ._metar_parser.metar_parser import parse, ParseError, TreeNode
 from ._tools import open_as_needed
 from .station_data import station_info
 from ..package_tools import Exporter
@@ -21,11 +21,12 @@ exporter = Exporter(globals())
 
 # Configure the named tuple used for storing METAR data
 Metar = namedtuple('metar', ['station_id', 'latitude', 'longitude', 'elevation', 'date_time',
-                             'wind_direction', 'wind_speed', 'visibility', 'current_wx1',
-                             'current_wx2', 'current_wx3', 'skyc1', 'skylev1', 'skyc2',
-                             'skylev2', 'skyc3', 'skylev3', 'skyc4', 'skylev4', 'cloudcover',
-                             'temperature', 'dewpoint', 'altimeter', 'current_wx1_symbol',
-                             'current_wx2_symbol', 'current_wx3_symbol', 'remarks'])
+                             'wind_direction', 'wind_speed', 'wind_gust', 'visibility',
+                             'current_wx1', 'current_wx2', 'current_wx3', 'skyc1', 'skylev1',
+                             'skyc2', 'skylev2', 'skyc3', 'skylev3', 'skyc4', 'skylev4',
+                             'cloudcover', 'temperature', 'dewpoint', 'altimeter',
+                             'current_wx1_symbol', 'current_wx2_symbol', 'current_wx3_symbol',
+                             'remarks'])
 
 # Create a dictionary for attaching units to the different variables
 col_units = {'station_id': None,
@@ -35,6 +36,7 @@ col_units = {'station_id': None,
              'date_time': None,
              'wind_direction': 'degrees',
              'wind_speed': 'kts',
+             'wind_gust': 'kts',
              'visibility': 'meters',
              'eastward_wind': 'kts',
              'northward_wind': 'kts',
@@ -59,6 +61,23 @@ col_units = {'station_id': None,
              'current_wx3_symbol': None}
 
 
+def _tree_repr_(self):
+    """Produce string representation of a TreeNodex object."""
+    rep = self.__class__.__name__ + '('
+    args = []
+    for key, value in self.__dict__.items():
+        if key == 'elements':
+            continue
+        args.append(key + '=' + repr(value))
+        dict_str = ', '.join(args) + ')'
+    rep += dict_str
+    return rep
+
+
+# Monkey patch to improve debugging
+TreeNode.__repr__ = _tree_repr_
+
+
 @exporter.export
 def parse_metar_to_dataframe(metar_text, *, year=None, month=None):
     """Parse a single METAR report into a Pandas DataFrame.
@@ -117,6 +136,7 @@ def parse_metar(metar_text, year, month, station_metadata=station_info):
     * 'date_time': Date and time of the observation, datetime object
     * 'wind_direction': Direction the wind is coming from, measured in degrees
     * 'wind_speed': Wind speed, measured in knots
+    * 'wind_gust': Wind gust, measured in knots
     * 'current_wx1': Current weather (1 of 3)
     * 'current_wx2': Current weather (2 of 3)
     * 'current_wx3': Current weather (3 of 3)
@@ -175,6 +195,7 @@ def parse_metar(metar_text, year, month, station_metadata=station_info):
         date_time = np.nan
 
     # Set the wind values
+    wind_units = 'kts'
     try:
         # If there are missing wind values, set wind speed and wind direction to nan
         if ('/' in tree.wind.text) or (tree.wind.text == 'KT') or (tree.wind.text == ''):
@@ -184,7 +205,8 @@ def parse_metar(metar_text, year, month, station_metadata=station_info):
         else:
             wind_spd = float(tree.wind.wind_spd.text)
             if 'MPS' in tree.wind.text:
-                wind_spd = units.Quantity(wind_spd, 'm/s').m_as('knots')
+                wind_units = 'm/s'
+                wind_spd = units.Quantity(wind_spd, wind_units).m_as('knots')
             if (tree.wind.wind_dir.text == 'VRB') or (tree.wind.wind_dir.text == 'VAR'):
                 wind_dir = np.nan
             else:
@@ -194,32 +216,43 @@ def parse_metar(metar_text, year, month, station_metadata=station_info):
         wind_dir = np.nan
         wind_spd = np.nan
 
-    # Handle visibility
-    if tree.vis.text.endswith('SM'):
-        visibility = 0
-        # Strip off the SM and any whitespace around the value and any leading 'M'
-        vis_str = tree.vis.text[:-2].strip().lstrip('M')
-
-        # Case of e.g. 1 1/4SM
-        if ' ' in vis_str:
-            whole, vis_str = vis_str.split(maxsplit=1)
-            visibility += int(whole)
-
-        # Handle fraction regardless
-        if '/' in vis_str:
-            num, denom = vis_str.split('/', maxsplit=1)
-            visibility += int(num) / int(denom)
-        else:  # Should be getting all cases of whole number without fraction
-            visibility += int(vis_str)
-        visibility = units.Quantity(visibility, 'miles').m_as('meter')
-    # CAVOK means vis is "at least 10km" and no significant clouds or weather
-    elif 'CAVOK' in tree.vis.text:
-        visibility = 10000
-    elif not tree.vis.text or tree.vis.text.strip() == '////':
-        visibility = np.nan
+    # Parse out the wind gust field
+    if 'G' in tree.wind.text:
+        wind_gust = units.Quantity(float(tree.wind.gust.text.strip()[1:]),
+                                   wind_units).m_as('knots')
     else:
-        # Only worry about the first 4 characters (digits) and ignore possible 'NDV'
-        visibility = int(tree.vis.text.strip()[:4])
+        wind_gust = np.nan
+
+    # Handle visibility
+    try:
+        if tree.vis.text.endswith('SM'):
+            visibility = 0
+            # Strip off the SM and any whitespace around the value and any leading 'M'
+            vis_str = tree.vis.text[:-2].strip().lstrip('M')
+
+            # Case of e.g. 1 1/4SM
+            if ' ' in vis_str:
+                whole, vis_str = vis_str.split(maxsplit=1)
+                visibility += int(whole)
+
+            # Handle fraction regardless
+            if '/' in vis_str:
+                num, denom = vis_str.split('/', maxsplit=1)
+                visibility += int(num) / int(denom)
+            else:  # Should be getting all cases of whole number without fraction
+                visibility += int(vis_str)
+            visibility = units.Quantity(visibility, 'miles').m_as('meter')
+        # CAVOK means vis is "at least 10km" and no significant clouds or weather
+        elif 'CAVOK' in tree.vis.text:
+            visibility = 10000
+        elif not tree.vis.text or tree.vis.text.strip() == '////':
+            visibility = np.nan
+        else:
+            # Only worry about the first 4 characters (digits) and ignore possible 'NDV'
+            visibility = int(tree.vis.text.strip()[:4])
+    # If there are any errors, return nan
+    except ValueError:
+        visibility = np.nan
 
     # Set the weather symbols
     # If the weather symbol is missing, set values to nan
@@ -245,7 +278,7 @@ def parse_metar(metar_text, year, month, station_metadata=station_info):
     if tree.skyc.text[1:3] == 'VV':
         skyc[0] = 'VV'
         level = tree.skyc.text.strip()[2:5]
-        skylev[0] = np.nan if '/' in level else 100 * int(level)
+        skylev[0] = np.nan if not level or '/' in level else 100 * int(level)
     else:
         for ind, part in enumerate(tree.skyc.text.strip().split(maxsplit=3)):
             cover = part[:3]
@@ -298,10 +331,10 @@ def parse_metar(metar_text, year, month, station_metadata=station_info):
         remarks = remarks[3:].strip()
 
     # Returns a named tuple with all the relevant variables
-    return Metar(station_id, lat, lon, elev, date_time, wind_dir, wind_spd, visibility,
-                 current_wx[0], current_wx[1], current_wx[2], skyc[0], skylev[0], skyc[1],
-                 skylev[1], skyc[2], skylev[2], skyc[3], skylev[3], cloudcover, temp, dewp,
-                 altim, current_wx_symbol[0], current_wx_symbol[1], current_wx_symbol[2],
+    return Metar(station_id, lat, lon, elev, date_time, wind_dir, wind_spd, wind_gust,
+                 visibility, current_wx[0], current_wx[1], current_wx[2], skyc[0], skylev[0],
+                 skyc[1], skylev[1], skyc[2], skylev[2], skyc[3], skylev[3], cloudcover, temp,
+                 dewp, altim, current_wx_symbol[0], current_wx_symbol[1], current_wx_symbol[2],
                  remarks)
 
 
@@ -363,6 +396,7 @@ def _metars_to_dataframe(metar_iter, *, year=None, month=None):
     * 'date_time': Date and time of the observation, datetime object
     * 'wind_direction': Direction the wind is coming from, measured in degrees
     * 'wind_speed': Wind speed, measured in knots
+    * 'wind_gust': Wind gust, measured in knots
     * 'visibility': Visibility distance, measured in meters
     * 'current_wx1': Current weather (1 of 3)
     * 'current_wx2': Current weather (2 of 3)
