@@ -66,7 +66,7 @@ def _read_master_text_file(input_file=None):
             lat = to_dec_deg(line[39:46].strip())
             lon = to_dec_deg(line[47:55].strip())
             alt_part = line[55:60].strip()
-            alt = int(alt_part if alt_part else 0.)
+            alt = int(alt_part or 0.)
             if stid:
                 if stid[0] in ('P', 'K'):
                     country = 'US'
@@ -110,16 +110,13 @@ def _read_airports_file(input_file=None):
     if input_file is None:
         input_file = get_test_data('airport-codes.csv', as_file_obj=False)
     df = pd.read_csv(input_file)
-    station_map = pd.DataFrame({'id': df.ident.values, 'synop_id': 99999,
-                                'latitude': df.latitude_deg.values,
-                                'longitude': df.longitude_deg.values,
-                                'altitude': units.Quantity(
-                                    df.elevation_ft.values, 'ft').to('m').m,
-                                'country': df.iso_region.str.split('-', n=1,
-                                                                   expand=True)[1].values,
-                                'source': input_file
-                                }).to_dict()
-    return station_map
+    return pd.DataFrame({'id': df.ident.values, 'synop_id': 99999,
+                         'latitude': df.latitude_deg.values,
+                         'longitude': df.longitude_deg.values,
+                         'altitude': units.Quantity(df.elevation_ft.values, 'ft').to('m').m,
+                         'country': df.iso_region.str.split('-', n=1, expand=True)[1].values,
+                         'source': input_file
+                         }).to_dict()
 
 
 class StationLookup:
@@ -149,7 +146,7 @@ with exporter:
 
 
 @exporter.export
-def add_station_lat_lon(df, stn_var):
+def add_station_lat_lon(df, stn_var=None):
     """Lookup station information to add the station latitude and longitude to the DataFrame.
 
     This function will add two columns to the DataFrame ('latitude' and 'longitude') after
@@ -159,17 +156,28 @@ def add_station_lat_lon(df, stn_var):
     ----------
     df : `pandas.DataFrame`
         The DataFrame that contains the station observations
-    stn_var : str
-        The string of the variable name that represents the station in the DataFrame. Common
-        examples are 'station', 'stid', and 'station_id'
+    stn_var : str, optional
+        The string of the variable name that represents the station in the DataFrame. If not
+        provided, 'station', 'stid', and 'station_id' are tried in that order.
 
     Returns
     -------
     `pandas.DataFrame` that contains original Dataframe now with the latitude and longitude
     values for each location found in `station_info`.
     """
+
+    def key_finder(df):
+        names_to_try = ('station', 'stid', 'station_id')
+        for id_name in names_to_try:
+            if id_name in df:
+                return id_name
+        raise KeyError('Second argument not provided to add_station_lat_lon, but none of '
+                       f'{names_to_try} were found.')
+
     df['latitude'] = None
     df['longitude'] = None
+    if stn_var is None:
+        stn_var = key_finder(df)
     for stn in df[stn_var].unique():
         try:
             info = station_info[stn]
