@@ -6,6 +6,7 @@ import collections
 import contextlib
 import copy
 from datetime import datetime, timedelta
+from difflib import get_close_matches
 from itertools import cycle
 import re
 
@@ -20,7 +21,7 @@ from . import ctables, wx_symbols
 from ._mpl import TextCollection
 from .cartopy_utils import import_cartopy
 from .station_plot import StationPlot
-from ..calc import reduce_point_density
+from ..calc import reduce_point_density, smooth_n_point, zoom_xarray
 from ..package_tools import Exporter
 from ..units import units
 
@@ -498,6 +499,43 @@ def lookup_map_feature(feature_name):
     return feat.with_scale(scaler)
 
 
+def plot_kwargs(data):
+    """Set the keyword arguments for MapPanel plotting."""
+    if hasattr(data.metpy, 'cartopy_crs'):
+        # Conditionally add cartopy transform if we are on a map.
+        kwargs = {'transform': data.metpy.cartopy_crs}
+    else:
+        kwargs = {}
+    return kwargs
+
+
+class ValidationMixin:
+    """Provides validation of attribute names when set by user."""
+
+    def __setattr__(self, name, value):
+        """Set only permitted attributes."""
+        allowlist = ['ax',
+                     'data',
+                     'handle',
+                     'notify_change',
+                     'panel'
+                     ]
+
+        allowlist.extend(self.trait_names())
+        if name in allowlist or name.startswith('_'):
+            super().__setattr__(name, value)
+        else:
+            closest = get_close_matches(name, allowlist, n=1)
+            if closest:
+                alt = closest[0]
+                suggest = f" Perhaps you meant '{alt}'?"
+            else:
+                suggest = ''
+            obj = self.__class__
+            msg = f"'{name}' is not a valid attribute for {obj}." + suggest
+            raise AttributeError(msg)
+
+
 class MetPyHasTraits(HasTraits):
     """Provides modification layer on HasTraits for declarative classes."""
 
@@ -512,14 +550,9 @@ class MetPyHasTraits(HasTraits):
 class Panel(MetPyHasTraits):
     """Draw one or more plots."""
 
-    @property
-    def plot_kwargs(self):
-        """Set the keyword arguments for MapPanel plotting."""
-        return {}
-
 
 @exporter.export
-class PanelContainer(MetPyHasTraits):
+class PanelContainer(MetPyHasTraits, ValidationMixin):
     """Collects panels and set complete figure related settings (e.g., size)."""
 
     size = Union([Tuple(Union([Int(), Float()]), Union([Int(), Float()])),
@@ -595,7 +628,7 @@ class PanelContainer(MetPyHasTraits):
 
 
 @exporter.export
-class MapPanel(Panel):
+class MapPanel(Panel, ValidationMixin):
     """Set figure related elements for an individual panel.
 
     Parameters that need to be set include collecting all plotting types
@@ -753,11 +786,7 @@ class MapPanel(Panel):
 
         """
         for item in self.layers:
-            if isinstance(item, str):
-                feat = lookup_map_feature(item)
-            else:
-                feat = item
-
+            feat = lookup_map_feature(item) if isinstance(item, str) else item
             yield feat
 
     @observe('area')
@@ -805,15 +834,6 @@ class MapPanel(Panel):
 
         return self._ax
 
-    @property
-    def plot_kwargs(self):
-        """Set the keyword arguments for MapPanel plotting."""
-        if isinstance(self.plots[0].griddata, tuple):
-            dataproj = self.plots[0].griddata[0].metpy.cartopy_crs
-        else:
-            dataproj = self.plots[0].griddata.metpy.cartopy_crs
-        return {'transform': dataproj}
-
     @ax.setter
     def ax(self, val):
         """Set the :class:`matplotlib.axes.Axes` to draw on.
@@ -954,7 +974,7 @@ class Plots2D(SubsetTraits):
 
     This attribute will scale the field by multiplying by the scale. For example, to
     scale vorticity to be whole values for contouring you could set the scale to 1e5, such that
-    the data values will be scaled by 10^5.
+    the data values will be multiplied by 10^5.
     """
 
     @property
@@ -1034,7 +1054,7 @@ class Plots2D(SubsetTraits):
         """Generate a name for the plot."""
         if isinstance(self.field, tuple):
             ret = ''
-            ret += ' and '.join(f for f in self.field)
+            ret += ' and '.join(self.field)
         else:
             ret = self.field
         if self.level is not None:
@@ -1066,6 +1086,37 @@ class PlotScalar(Plots2D):
     `list(ds)`
     """
 
+    smooth_field = Int(allow_none=True, default_value=None)
+    smooth_field.__doc__ = """Number of smoothing passes using 9-pt smoother.
+
+    By setting this parameter with an integer value it will call the MetPy 9-pt smoother and
+    provide a smoothed field for plotting. It is best to use this smoothing for data with
+    finer resolutions (e.g., smaller grid spacings with a lot of grid points).
+
+    See Also
+    --------
+    metpy.calc.smooth_n_point, smooth_contour
+    """
+
+    smooth_contour = Union([Int(allow_none=True, default_value=None),
+                            Tuple(Int(allow_none=True, default_value=None),
+                                  Int(allow_none=True, default_value=None))])
+    smooth_contour.__doc__ = """Spline interpolation to smooth contours.
+
+    This attribute requires settings for the `metpy.calc.zoom_xarray` function, which will
+    produce a spline interpolation given an integer zoom factor. Either a single integer
+    specifying the zoom factor (e.g., 4) or a tuple containing two integers for the zoom factor
+    and the spline interpolation order can be used. The default spline interpolation order is
+    3.
+
+    This is best used to smooth contours when contouring a sparse grid (e.g., when your data
+    has a large grid spacing).
+
+    See Also
+    --------
+    metpy.calc.zoom_xarray, smooth_field
+    """
+
     @observe('field')
     def _update_data(self, _=None):
         """Handle updating the internal cache of data.
@@ -1103,6 +1154,19 @@ class PlotScalar(Plots2D):
                     'for plotting'
                 )
 
+            # Handle smoothing of data
+            if self.smooth_field is not None:
+                data_subset = smooth_n_point(data_subset, 9, self.smooth_field)
+            # Handle zoom interpolation
+            if self.smooth_contour is not None:
+                if isinstance(self.smooth_contour, tuple):
+                    zoom = self.smooth_contour[0]
+                    order = self.smooth_contour[1]
+                else:
+                    zoom = self.smooth_contour
+                    order = 3
+                data_subset = zoom_xarray(data_subset, zoom, order=order)
+
             # Handle unit conversion (both direct unit specification and scaling)
             if self.plot_units is not None:
                 data_subset = data_subset.metpy.convert_units(self.plot_units)
@@ -1145,13 +1209,14 @@ class PlotScalar(Plots2D):
 class ContourTraits(MetPyHasTraits):
     """Represents common contour traits."""
 
-    contours = Union([List(Float()), Int()], default_value=25)
+    contours = Union([List(Float()), Int(), Instance(range)], default_value=25)
     contours.__doc__ = """A list of values to contour or an integer number of contour levels.
 
     This parameter sets contour or colorfill values for a plot. Values can be entered either
-    as a list of values or as an integer with the number of contours to be plotted (as per
-    matplotlib documentation). A list can be generated by using square brackets or creating
-    a numpy 1D array and converting it to a list with the `~numpy.ndarray.tolist` method.
+    as a Python range instance, a list of values or as an integer with the number of contours
+    to be plotted (as per matplotlib documentation). A list can be generated by using square
+    brackets or creating a numpy 1D array and converting it to a list with the
+    `~numpy.ndarray.tolist` method.
     """
 
     clabels = Bool(default_value=False)
@@ -1206,7 +1271,7 @@ class ColorfillTraits(MetPyHasTraits):
 
 
 @exporter.export
-class ImagePlot(PlotScalar, ColorfillTraits):
+class ImagePlot(PlotScalar, ColorfillTraits, ValidationMixin):
     """Make raster image using `~matplotlib.pyplot.imshow` for satellite or colored image."""
 
     @observe('colormap', 'image_range')
@@ -1244,7 +1309,7 @@ class ImagePlot(PlotScalar, ColorfillTraits):
         """Build the plot by calling any plotting methods as necessary."""
         x_like, y_like, imdata = self.plotdata
 
-        kwargs = self.parent.plot_kwargs
+        kwargs = plot_kwargs(imdata)
 
         # If we're on a map, we use min/max for y and manually figure out origin to try to
         # avoid upside down images created by images where y[0] > y[-1], as well as
@@ -1261,7 +1326,7 @@ class ImagePlot(PlotScalar, ColorfillTraits):
 
 
 @exporter.export
-class ContourPlot(PlotScalar, ContourTraits):
+class ContourPlot(PlotScalar, ContourTraits, ValidationMixin):
     """Make contour plots by defining specific traits."""
 
     linecolor = Unicode('black')
@@ -1299,7 +1364,7 @@ class ContourPlot(PlotScalar, ContourTraits):
         """Build the plot by calling any plotting methods as necessary."""
         x_like, y_like, imdata = self.plotdata
 
-        kwargs = self.parent.plot_kwargs
+        kwargs = plot_kwargs(imdata)
 
         self.handle = self.parent.ax.contour(x_like, y_like, imdata, self.contours,
                                              colors=self.linecolor, linewidths=self.linewidth,
@@ -1310,7 +1375,7 @@ class ContourPlot(PlotScalar, ContourTraits):
 
 
 @exporter.export
-class FilledContourPlot(PlotScalar, ColorfillTraits, ContourTraits):
+class FilledContourPlot(PlotScalar, ColorfillTraits, ContourTraits, ValidationMixin):
     """Make color-filled contours plots by defining appropriate traits."""
 
     @observe('contours', 'colorbar', 'colormap')
@@ -1324,7 +1389,7 @@ class FilledContourPlot(PlotScalar, ColorfillTraits, ContourTraits):
         """Build the plot by calling any plotting methods as necessary."""
         x_like, y_like, imdata = self.plotdata
 
-        kwargs = self.parent.plot_kwargs
+        kwargs = plot_kwargs(imdata)
 
         self.handle = self.parent.ax.contourf(x_like, y_like, imdata, self.contours,
                                               cmap=self._cmap_obj, norm=self._norm_obj,
@@ -1400,12 +1465,12 @@ class PlotVector(Plots2D):
         """Return the internal cached data."""
         if getattr(self, '_griddata_u', None) is None:
 
-            if self.field[0]:
-                u = self.data.metpy.parse_cf(self.field[0])
-                v = self.data.metpy.parse_cf(self.field[1])
-            else:
+            if not self.field[0]:
                 raise ValueError('field attribute not set correctly')
 
+            u = self.data.metpy.parse_cf(self.field[0])
+            v = self.data.metpy.parse_cf(self.field[1])
+
             # Subset to 2D using MetPy's fancy .sel
             subset = {'method': 'nearest'}
             for dim_coord in ('x', 'longitude', 'y', 'latitude', 'vertical', 'time'):
@@ -1423,8 +1488,8 @@ class PlotVector(Plots2D):
             if self.plot_units is not None:
                 data_subset_u = data_subset_u.metpy.convert_units(self.plot_units)
                 data_subset_v = data_subset_v.metpy.convert_units(self.plot_units)
-            self._griddata_u = data_subset_u
-            self._griddata_v = data_subset_v
+            self._griddata_u = data_subset_u * self.scale
+            self._griddata_v = data_subset_v * self.scale
 
         return (self._griddata_u, self._griddata_v)
 
@@ -1482,7 +1547,7 @@ class PlotVector(Plots2D):
 
 
 @exporter.export
-class BarbPlot(PlotVector):
+class BarbPlot(PlotVector, ValidationMixin):
     """Make plots of wind barbs on a map with traits to refine the look of plotted elements."""
 
     barblength = Float(default_value=7)
@@ -1503,7 +1568,7 @@ class BarbPlot(PlotVector):
         """Build the plot by calling needed plotting methods as necessary."""
         x_like, y_like, u, v = self.plotdata
 
-        kwargs = self.parent.plot_kwargs
+        kwargs = plot_kwargs(u)
 
         # Conditionally apply the proper transform
         if 'transform' in kwargs and self.earth_relative:
@@ -1518,7 +1583,7 @@ class BarbPlot(PlotVector):
 
 
 @exporter.export
-class PlotObs(MetPyHasTraits):
+class PlotObs(MetPyHasTraits, ValidationMixin):
     """The highest level class related to plotting observed surface and upperair data.
 
     This class collects all common methods no matter whether plotting a upper-level or
@@ -1680,7 +1745,7 @@ class PlotObs(MetPyHasTraits):
     def name(self):
         """Generate a name for the plot."""
         ret = ''
-        ret += ' and '.join(f for f in self.fields)
+        ret += ' and '.join(self.fields)
         if self.level is not None:
             ret += f'@{self.level:d}'
         return ret
@@ -1778,10 +1843,7 @@ class PlotObs(MetPyHasTraits):
 
         # Use the cartopy map projection to transform station locations to the map and
         # then refine the number of stations plotted by setting a radius
-        if self.parent._proj_obj == ccrs.PlateCarree():
-            scale = 1.
-        else:
-            scale = 100000.
+        scale = 1. if self.parent._proj_obj == ccrs.PlateCarree() else 100000.
         point_locs = self.parent._proj_obj.transform_points(ccrs.PlateCarree(), lon, lat)
         subset = reduce_point_density(point_locs, self.reduce_points * scale)
 
@@ -1790,10 +1852,7 @@ class PlotObs(MetPyHasTraits):
 
         for i, ob_type in enumerate(self.fields):
             field_kwargs = {}
-            if len(self.locations) > 1:
-                location = self.locations[i]
-            else:
-                location = self.locations[0]
+            location = self.locations[i] if len(self.locations) > 1 else self.locations[0]
             if len(self.colors) > 1:
                 field_kwargs['color'] = self.colors[i]
             else:
@@ -1826,9 +1885,11 @@ class PlotObs(MetPyHasTraits):
                 self.handle.plot_parameter(location, parameter, **field_kwargs)
 
         if self.vector_field[0] is not None:
-            vector_kwargs = {}
-            vector_kwargs['color'] = self.vector_field_color
-            vector_kwargs['plot_units'] = self.vector_plot_units
+            vector_kwargs = {
+                'color': self.vector_field_color,
+                'plot_units': self.vector_plot_units,
+            }
+
             if hasattr(self.data, 'units') and (vector_kwargs['plot_units'] is not None):
                 u = units.Quantity(data[self.vector_field[0]][subset].values,
                                    self.data.units[self.vector_field[0]])
@@ -2007,9 +2068,7 @@ class PlotGeometry(MetPyHasTraits):
         else:
             coords = geo_obj.coords
 
-        position = coords[label_hash % len(coords)]
-
-        return position
+        return coords[label_hash % len(coords)]
 
     def _draw_label(self, text, lon, lat, color='black', outline='white', offset=(0, 0)):
         """Draw a label to the plot.
@@ -2030,15 +2089,15 @@ class PlotGeometry(MetPyHasTraits):
             A tuple containing the x- and y-offset of the label, respectively
         """
         path_effects = [patheffects.withStroke(linewidth=4, foreground=outline)]
-        self.parent.ax.add_collection(TextCollection([lon], [lat], [str(text)],
-                                                     va='center',
-                                                     ha='center',
-                                                     offset=offset,
-                                                     weight='demi',
-                                                     size=self.label_fontsize,
-                                                     color=color,
-                                                     path_effects=path_effects,
-                                                     transform=ccrs.PlateCarree()))
+        self.parent.ax.add_artist(TextCollection([lon], [lat], [str(text)],
+                                                 va='center',
+                                                 ha='center',
+                                                 offset=offset,
+                                                 weight='demi',
+                                                 size=self.label_fontsize,
+                                                 color=color,
+                                                 path_effects=path_effects,
+                                                 transform=ccrs.PlateCarree()))
 
     def draw(self):
         """Draw the plot."""
