@@ -28,7 +28,7 @@ from pyproj import CRS, Proj
 import xarray as xr
 
 from ._vendor.xarray import either_dict_or_kwargs, expanded_indexer, is_dict_like
-from .units import DimensionalityError, UndefinedUnitError, units
+from .units import _mutate_arguments, DimensionalityError, UndefinedUnitError, units
 
 __all__ = ('MetPyDataArrayAccessor', 'MetPyDatasetAccessor', 'grid_deltas_from_dataarray')
 metpy_axes = ['time', 'vertical', 'y', 'latitude', 'x', 'longitude']
@@ -159,15 +159,16 @@ class MetPyDataArrayAccessor:
     def convert_units(self, units):
         """Return new DataArray with values converted to different units.
 
+        See Also
+        --------
+        convert_coordinate_units
+
         Notes
         -----
         Any cached/lazy-loaded data (except that in a Dask array) will be loaded into memory
         by this operation. Do not utilize on moderate- to large-sized remote datasets before
         subsetting!
 
-        See Also
-        --------
-        convert_coordinate_units
         """
         return self.quantify().copy(data=self.unit_array.to(units))
 
@@ -177,14 +178,15 @@ class MetPyDataArrayAccessor:
         This operation differs from ``.convert_units`` since xarray coordinate indexes do not
         yet support unit-aware arrays (even though unit-aware *data* arrays are).
 
+        See Also
+        --------
+        convert_units
+
         Notes
         -----
         Any cached/lazy-loaded coordinate data (except that in a Dask array) will be loaded
         into memory by this operation.
 
-        See Also
-        --------
-        convert_units
         """
         new_coord_var = self._data_array[coord].copy(
             data=self._data_array[coord].metpy.unit_array.m_as(units)
@@ -230,7 +232,9 @@ class MetPyDataArrayAccessor:
         """Return the coordinate reference system (CRS) as a CFProjection object."""
         if 'metpy_crs' in self._data_array.coords:
             return self._data_array.coords['metpy_crs'].item()
-        raise AttributeError('crs attribute is not available.')
+        raise AttributeError('crs attribute is not available. You may need to use the'
+                             ' `parse_cf` or `assign_crs` methods. Consult the "xarray'
+                             ' with MetPy Tutorial" for more details.')
 
     @property
     def cartopy_crs(self):
@@ -366,9 +370,10 @@ class MetPyDataArrayAccessor:
         # coordinates, and nothing else.
         coord_map = self._generate_coordinate_map()
         for axis, coord_var in coord_map.items():
-            if (coord_var is not None
-                and not any(axis in coord.attrs.get('_metpy_axis', '').split(',')
-                            for coord in coords)):
+            if coord_var is not None and all(
+                axis not in coord.attrs.get('_metpy_axis', '').split(',')
+                for coord in coords
+            ):
 
                 _assign_axis(coord_var.attrs, axis)
 
@@ -377,15 +382,15 @@ class MetPyDataArrayAccessor:
 
     def _axis(self, axis):
         """Return the coordinate variable corresponding to the given individual axis type."""
-        if axis in metpy_axes:
-            coord_var = self._metpy_axis_search(axis)
-            if coord_var is not None:
-                return coord_var
-            else:
-                raise AttributeError(axis + ' attribute is not available.')
-        else:
+        if axis not in metpy_axes:
             raise AttributeError("'" + axis + "' is not an interpretable axis.")
 
+        coord_var = self._metpy_axis_search(axis)
+        if coord_var is None:
+            raise AttributeError(axis + ' attribute is not available.')
+        else:
+            return coord_var
+
     def coordinates(self, *args):
         """Return the coordinate variables corresponding to the given axes types.
 
@@ -742,8 +747,8 @@ class MetPyDatasetAccessor:
                 proj_var = self._dataset.variables[proj_name]
             except KeyError:
                 log.warning(
-                    'Could not find variable corresponding to the value of '
-                    f'grid_mapping: {proj_name}')
+                    'Could not find variable corresponding to the value of grid_mapping: %s',
+                    proj_name)
             else:
                 crs = CFProjection(proj_var.attrs)
 
@@ -767,8 +772,8 @@ class MetPyDatasetAccessor:
             else:
                 if latitude.identical(y) and longitude.identical(x):
                     crs = CFProjection({'grid_mapping_name': 'latitude_longitude'})
-                    log.warning('Found valid latitude/longitude coordinates, assuming '
-                                'latitude_longitude for projection grid_mapping variable')
+                    log.debug('Found valid latitude/longitude coordinates, assuming '
+                              'latitude_longitude for projection grid_mapping variable')
 
         # Rebuild the coordinates of the dataarray, and return quantified DataArray
         var = self._rebuild_coords(var, crs)
@@ -843,15 +848,15 @@ class MetPyDatasetAccessor:
         `xarray.Dataset`
             New xarray Dataset with CRS coordinate assigned
 
+        See Also
+        --------
+        parse_cf
+
         Notes
         -----
         CF projection arguments should be supplied as a dictionary or collection of kwargs,
         but not both.
 
-        See Also
-        --------
-        parse_cf
-
         """
         return _assign_crs(self._dataset, cf_attributes, kwargs)
 
@@ -1166,9 +1171,18 @@ def preprocess_and_wrap(broadcast=None, wrap_like=None, match_unit=False, to_mag
         xarray arguments to Quantity, and do not change other array-like arguments.
     """
     def decorator(func):
+        sig = signature(func)
+        if broadcast is not None:
+            for arg_name in broadcast:
+                if arg_name not in sig.parameters:
+                    raise ValueError(
+                        f'Cannot broadcast argument {arg_name} as it is not in function '
+                        'signature'
+                    )
+
         @functools.wraps(func)
         def wrapper(*args, **kwargs):
-            bound_args = signature(func).bind(*args, **kwargs)
+            bound_args = sig.bind(*args, **kwargs)
 
             # Auto-broadcast select xarray arguments, and update bound_args
             if broadcast is not None:
@@ -1232,22 +1246,6 @@ def preprocess_and_wrap(broadcast=None, wrap_like=None, match_unit=False, to_mag
     return decorator
 
 
-def _mutate_arguments(bound_args, check_type, mutate_arg):
-    """Handle adjusting bound arguments.
-
-    Calls ``mutate_arg`` on every argument, including those passed as ``*args``, if they are
-    of type ``check_type``.
-    """
-    for arg_name, arg_val in bound_args.arguments.items():
-        if isinstance(arg_val, check_type):
-            bound_args.arguments[arg_name] = mutate_arg(arg_val, arg_name)
-
-    if isinstance(bound_args.arguments.get('args'), tuple):
-        bound_args.arguments['args'] = tuple(
-            mutate_arg(arg_val, '(unnamed)') if isinstance(arg_val, check_type) else arg_val
-            for arg_val in bound_args.arguments['args'])
-
-
 def _wrap_output_like_matching_units(result, match):
     """Convert result to be like match with matching units for output wrapper."""
     output_xarray = isinstance(match, xr.DataArray)
@@ -1272,20 +1270,19 @@ def _wrap_output_like_not_matching_units(result, match):
     output_xarray = isinstance(match, xr.DataArray)
     if isinstance(result, xr.DataArray):
         return result if output_xarray else result.metpy.unit_array
-    else:
-        # Determine if need to upcast to Quantity
-        if (
-            not isinstance(result, units.Quantity)
-            and (
-                isinstance(match, units.Quantity)
-                or (output_xarray and isinstance(match.data, units.Quantity))
-            )
-        ):
-            result = units.Quantity(result)
-        return (
-            xr.DataArray(result, coords=match.coords, dims=match.dims) if output_xarray
-            else result
+    # Determine if need to upcast to Quantity
+    if (
+        not isinstance(result, units.Quantity)
+        and (
+            isinstance(match, units.Quantity)
+            or (output_xarray and isinstance(match.data, units.Quantity))
         )
+    ):
+        result = units.Quantity(result)
+    return (
+        xr.DataArray(result, coords=match.coords, dims=match.dims) if output_xarray
+        else result
+    )
 
 
 def check_matching_coordinates(func):
